--
-- EPP if testbench
--

LIBRARY ieee;
library work;
USE ieee.STD_LOGIC_1164.ALL;
USE ieee.numeric_std.all;
use work.epp_if_pkg.all;

entity epp_if_tb is
end epp_if_tb;

architecture TB of epp_if_tb is

constant PERIOD : time := 1 sec/100e6;

signal clock : std_logic := '0';
signal reset : std_logic := '1';

signal data : std_logic_vector(7 downto 0);

signal W_addr             : std_logic_vector(7 downto 0);
signal W_dout             : std_logic_vector(7 downto 0);
signal W_din              : std_logic_vector(7 downto 0);
signal W_rd               : std_logic;
signal W_wr               : std_logic;
signal W_waitrequest      : std_logic;
signal W_reset_out        : std_logic;
signal W_addr_autoinc_ena : std_logic;

signal W_epp_data           :  std_logic_vector(7 downto 0);
signal W_epp_wait           :  std_logic;
signal W_epp_dir            :  std_logic;
signal W_epp_astrobeb       :  std_logic;
signal W_epp_dstrobeb       :  std_logic;
signal W_epp_reset          :  std_logic;

procedure epp_write
(
  signal epp_strobeb : out std_logic; 
  signal epp_dir : out std_logic;
  signal epp_wait : in std_logic;
  signal epp_dbus : inout std_logic_vector(7 downto 0); 
  data : std_logic_vector(7 downto 0)
);

procedure epp_read
(
  signal epp_strobeb : out std_logic; 
  signal epp_dir : out std_logic;
  signal epp_wait : in std_logic;
  signal epp_dbus : inout std_logic_vector(7 downto 0);
  signal dout : out std_logic_vector(7 downto 0)
);

procedure epp_write
(
  signal epp_strobeb : out std_logic; 
  signal epp_dir : out std_logic;
  signal epp_wait : in std_logic;
  signal epp_dbus : inout std_logic_vector(7 downto 0); 
  data : std_logic_vector(7 downto 0)
) is
begin
  epp_dir <= '0';
  epp_dbus <= data;
  epp_strobeb <= '0';
  wait until epp_wait = '1';
  wait for 10 ns;
  epp_strobeb <= '1';
  epp_dbus <= (others => 'Z');
  epp_dir <= '1';
end procedure;

procedure epp_read
(
  signal epp_strobeb : out std_logic; 
  signal epp_dir : out std_logic;
  signal epp_wait : in std_logic;
  signal epp_dbus : inout std_logic_vector(7 downto 0);
  signal dout : out std_logic_vector(7 downto 0)
) is
begin
  epp_dir <= '1';
  epp_dbus <= (others => 'Z');
  epp_strobeb <= '0';
  wait until epp_wait = '1';
  dout <= epp_dbus;
  epp_strobeb <= '1';
  epp_dir <= '1';
end procedure;

begin

DUT: epp_if
generic map
(
  WAITCOUNT => 10
)
port map
(
  clk => clock,
  reset => reset,
  --
  epp_data          => W_epp_data,
  epp_wait          => W_epp_wait,
  epp_dir_in        => W_epp_dir,
  epp_astrobeb_in   => W_epp_astrobeb,
  epp_dstrobeb_in   => W_epp_dstrobeb,
  epp_reset_in      => W_epp_reset,
  --
  addr              => W_addr,
  dout              => W_dout,
  din               => W_din,
  rd                => W_rd,
  wr                => W_wr,
  waitrequest       => W_waitrequest,
  reset_out         => W_reset_out,
  addr_autoinc_ena  => W_addr_autoinc_ena
);

process(clock)
begin
  clock <= not clock after PERIOD/2;
end process;

reset <= '1', '0' after 100 ns;

process(W_addr)
begin
  case W_addr is
    when x"12" => 
      W_din <= x"AA";
    when others => 
      W_din <= (others => 'X');
  end case;
end process;

process
begin
  W_epp_data <= (others => 'Z');
  W_epp_dir <= '0';
  W_epp_astrobeb <= '1';
  W_epp_dstrobeb <= '1';
  W_epp_reset <= '1';
  W_waitrequest <= '0';
  W_addr_autoinc_ena <= '0';
  wait until reset = '0';
  --
  epp_write(W_epp_astrobeb, W_epp_dir, W_epp_wait, W_epp_data, x"12");
  wait for 100 ns;
  epp_write(W_epp_dstrobeb, W_epp_dir, W_epp_wait, W_epp_data, x"34");
  wait for 100 ns;
  epp_read(W_epp_dstrobeb, W_epp_dir, W_epp_wait, W_epp_data, data);
  wait for 100 ns;
  epp_read(W_epp_astrobeb, W_epp_dir, W_epp_wait, W_epp_data, data);
  wait;
end process;

end;
