--
-- QEDEC.vhd
--
-- Quadrature encoder decoder
--
-- 2010-01-22 JTA
--

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

ENTITY qedec IS
PORT
	(
		clk : in std_logic;
		reset : in std_logic;
		--
		cha : in std_logic;
		chb : in std_logic;
		--
    inc : out std_logic;
    dec : out std_logic
    
	);
END qedec;

architecture JTA_design of qedec is

signal currstate : std_logic_vector(1 downto 0) := "00";
signal newstate : std_logic_vector(1 downto 0) := "00";

begin

newstate <= cha & chb;

--
-- Quadrature pulse decoder
--
-- The definition of direction is as follows:
--
-- "When the codewheel rotates in the counterclockwise direction (as viewed from the 
-- encoder end of the motor), channel A will lead channel B. If the codewheel rotates 
-- in the clockwise direction, channel B will lead channel A."
--
-- Clockwise direction increments counter and counterclockwise decrements, respectively.
--

process(currstate, newstate, reset)
begin
	if reset = '1' then
		inc <= '0';
		dec <= '0';
	else
		inc <= '0';
		dec <= '0';

		case currstate is
			when "00" =>				
				if newstate = "01" then
					inc <= '1';
				elsif newstate = "10" then
					dec <= '1';
				end if;
					
			when "01" =>
				if newstate = "11" then
					inc <= '1';
				elsif newstate = "00" then
					dec <= '1';
				end if;
			
			when "11" =>
				if newstate = "10" then
					inc <= '1';
				elsif newstate = "01" then
					dec <= '1';
				end if;
	
			when "10" =>
				if newstate = "00" then
					inc <= '1';
				elsif newstate = "11" then
					dec <= '1';
				end if;
	
			when others =>
		end case;	
	end if;
end process;

process(reset, clk, newstate)
begin
	if reset = '1' then
		currstate <= "00";
	elsif rising_edge(clk) then
		currstate <= newstate;
	end if;
end process;

end;