--
-- Rate generator
--
-- JTA 2010-09-25
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.rate_gen_pkg.all;

entity rate_gen is 
generic
(
  REGWIDTH : natural := 24;
  CLKDIV : natural := 10
);
port
(
  clk : in std_logic;
  reset : in std_logic;
  rate : in std_logic_vector(REGWIDTH-1 downto 0);
  enable : in std_logic;
  pout : out std_logic
);
end rate_gen;

architecture RTL of rate_gen is

signal clkdivcnt : integer range 0 to CLKDIV := 0;
signal tick : boolean := false;
signal ratecnt : unsigned(rate'range);

begin

process(clk, reset)
begin
  if reset = '1' then
    clkdivcnt <= 0;
    tick <= false;
  elsif rising_edge(clk) then
    tick <= false;
    if clkdivcnt < CLKDIV-1 then
      clkdivcnt <= clkdivcnt + 1;
    else
      clkdivcnt <= 0;
      tick <= true;
    end if;
  end if;
end process;

process(clk, reset)
begin
  if reset = '1' then
    ratecnt <= to_unsigned(0, ratecnt'length);
    pout <= '0';
  elsif rising_edge(clk) then
    pout <= '0';
    if enable = '1' then
      if tick then
        if ratecnt < unsigned(rate) then
          ratecnt <= ratecnt + 1;
        else
          pout <= '1';
          ratecnt <= to_unsigned(0, ratecnt'length);
        end if;        
      end if;
    else
      ratecnt <= to_unsigned(0, ratecnt'length);
    end if;
  end if;
end process;

end RTL;